<?php

declare(strict_types=1);

namespace app\controllers;

use yii;
use yii\web\Controller;
use yii\web\Response;
use yii\filters\AccessControl;

use app\models\Account;

class AccountController extends Controller
{
    public function behaviors()
    {
        return [
            'access' => [
                'class' => AccessControl::class,
                'rules' => [
                    [
                        'allow' => true,
                        'roles' => ['@'],
                        'actions' => [
                            'index',
                            'edit',
                        ]
                    ],
                    [
                        'allow' => false,
                        'roles' => ['?'],
                        'denyCallback' => [$this, 'accessDenied']
                    ]
                ]
            ]
        ];
    }

    public function accessDenied()
    {
        // Инициализация
        $cookies = yii::$app->response->cookies;

        // Запись cookie с редиректом, который выполнится после авторизации
        $cookies->add(new Cookie([
            'name' => 'redirect',
            'value' => yii::$app->request->pathInfo
        ]));

        if (Yii::$app->request->isPost) {
            // POST-запрос

            // Настройка
            Yii::$app->response->format = Response::FORMAT_JSON;

            // Генерация ответа
            Yii::$app->response->content = json_encode([
                'main' => $this->renderPartial('/account/index'),
                'redirect' => yii::$app->request->pathInfo,
                '_csrf' => Yii::$app->request->getCsrfToken()
            ]);
        } else if (Yii::$app->request->isGet) {
            // GET-запрос

            $this->redirect('/authentication');
        }
    }

    public function actionIndex()
    {
        return $this->renderPartial('/accounts/index');
    }

    public function actionEdit(int $_key, string $target)
    {
        if (yii::$app->user->isGuest) {
            // Аккаунт не аутентифицирован
        } else {
            if (yii::$app->request->isPost) {
                // AJAX-POST-запрос

                if ($account = Account::searchById(Account::collectionName() . '/' . $_key)) {
                    // Аккаунт найден

                    // Запись параметра
                    $account->{$target} = yii::$app->request->post('value') ?? yii::$app->request->get('value') ?? 'Неизвестно';

                    // Настройка формата ответа
                    yii::$app->response->format = Response::FORMAT_JSON;

                    return $account->update() > 0;
                }
            }
        }

        // Запись кода ответа
        yii::$app->response->statusCode = 500;

        return false;
    }
}
